@@ -1,29 +1,26 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "harness.h"
+#include "harness_graph.h"
 #include "tbb/flow_graph.h"
 #include "tbb/task_scheduler_init.h"
 
 #if defined(_MSC_VER) && _MSC_VER < 1600
-    #pragma warning (disable : 4503) //disabling the "decorated name length exceeded" warning for VS2008 and earlier
+    // #pragma warning (disable : 4503) //disabling the "decorated name length exceeded" warning for VS2008 and earlier
 #endif
 
 //
@@ -214,7 +211,6 @@ public:
     typedef tbb::flow::source_node<TType> source_type;
     static const int N = tbb::flow::tuple_size<TType>::value;
     static void test() {
-        TType v;
         source_type* all_source_nodes[MaxNSources];
         sink_node_helper<N,SType>::print_parallel_remark();
         REMARK(" >\n");
@@ -295,7 +291,9 @@ static void test() {
     SType* my_split = makeSplit<TUPLE_SIZE,SType>::create(g);
     sink_node_helper<TUPLE_SIZE, SType>::print_serial_remark(); REMARK(" >\n");
 
-    test_one_serial<SType>( *my_split, g);
+    test_output_ports_return_ref(*my_split);
+
+    test_one_serial<SType>(*my_split, g);
     // build the vector with copy construction from the used split node.
     std::vector<SType>split_vector(ELEMS, *my_split);
     // destroy the tired old split_node in case we're accidentally reusing pieces of it.
@@ -303,7 +301,7 @@ static void test() {
 
 
     for(int e = 0; e < ELEMS; ++e) {  // exercise each of the vector elements
-        test_one_serial<SType>( split_vector[e], g);
+        test_one_serial<SType>(split_vector[e], g);
     }
 }
 
